/*============================================================================
  File:     IsolationLevels.sql

  Summary:  This script shows the database level settings that can be changed
			to use a variety of different isolation settings, their combinations
			and their effects on data access.

  Date:     June 2005

  SQL Server Version: 9.00.1116 (IDW14)
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE AdventureWorks
go

------------------------------------------------------------------------------
-- DEFAULT Behavior - no changes to database
-- Verify that NONE of the snapshot options have been turned on
------------------------------------------------------------------------------

SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- Enable RSCI - Statement-level Read Committed with Row Versioning
------------------------------------------------------------------------------

ALTER DATABASE AdventureWorks
SET READ_COMMITTED_SNAPSHOT ON
WITH ROLLBACK AFTER 5

-- Why "WITH ROLLBACK AFTER"?
-- Because RCSI requires no application changes... huh?
-- All users run in this mode by default, when turned on. In order
-- to turn this on (thereby requiring no application changes), you
-- must have exclusive access to the database so that there's a defined
-- point in time to which versioning begins. ROLLBACK AFTER allows
-- users to continue for n more seconds and then, if still processing,
-- any remaining/pending transaction(s) will be rolled back.
-- Default is to wait indefinitely. Best to change this off hours.

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- DISABLE RSCI - Statement-level Read Committed with Row Versioning
------------------------------------------------------------------------------

ALTER DATABASE AdventureWorks
SET READ_COMMITTED_SNAPSHOT OFF
WITH ROLLBACK AFTER 5
go

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- Enable Snapshot Isolation - Transaction-level Consistency
------------------------------------------------------------------------------

-- This code needs to go to another window (paste all to rollback but only execute to stop)
--USE AdventureWorks
--go
--BEGIN TRAN
--	UPDATE Person.Contact SET FirstName = 'Kimberly' WHERE ContactID = 3

-- stop here

SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'

-- stop

--ROLLBACK


-- Now execute the change, this command will wait for the open trans to complete
ALTER DATABASE AdventureWorks
SET ALLOW_SNAPSHOT_ISOLATION ON
go

-- Go to a third window and request a snapshot transaction
--SET TRANSACTION ISOLATION LEVEL SNAPSHOT
--go
--
--BEGIN TRAN
--SELECT * FROM person.address

--Msg 3956, Level 16, State 1, Line 1
--Transaction failed in database 'AdventureWorks' because an ALTER DATABASE command 
--which enables snapshot isolation is not finished yet. Wait until the command is finished.

-- Go back to the first window and verify the setting of the database (3 - in transition)

-- You are in transition (3) - the alter database execution will wait.
-- You can use WITH ROLLBACK AFTER but connected users, not running
-- transactions will not block this statement
-- Regardless, always best to change this off hours.

-- then complete the transaction to allow the alter to complete.
-- ROLLBACK

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- DISABLE Snapshot Isolation - Transaction-level Consistency
------------------------------------------------------------------------------

ALTER DATABASE AdventureWorks
SET ALLOW_SNAPSHOT_ISOLATION OFF
go

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- Enable BOTH RCSI AND Snapshot Isolation
------------------------------------------------------------------------------

ALTER DATABASE AdventureWorks
SET READ_COMMITTED_SNAPSHOT ON
WITH ROLLBACK AFTER 5
go

ALTER DATABASE AdventureWorks
SET ALLOW_SNAPSHOT_ISOLATION ON
go

-- Note: These cannot be changed together. RCSI must have exclusive use.

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go

------------------------------------------------------------------------------
-- DISABLE Snapshot Isolation - Transaction-level Consistency
------------------------------------------------------------------------------

ALTER DATABASE AdventureWorks
SET READ_COMMITTED_SNAPSHOT OFF
WITH ROLLBACK AFTER 5
go

ALTER DATABASE AdventureWorks
SET ALLOW_SNAPSHOT_ISOLATION OFF
go

-- Verify setting
SELECT sd.[name] AS [Database Name]
	, sd.is_read_committed_snapshot_on AS [RCSI Enabled]
	, snapshot_isolation_state AS [Snaphot Isolation Enabled]
FROM sys.databases AS sd
WHERE sd.[name] = 'AdventureWorks'
go
